# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from scipy.optimize import fsolve


def real_eigenvalue(x, w, h):
    """Compute the real eigenvalue

    Args:
        x: initial guess for fsolve
        w: angular fequency
        h: mean water level

    Returns:
        Real eigenvalue
    """
    eigenvalue = ((w ** 2) * h / 9.8) / x - np.tanh(x)
    return eigenvalue


def complex_eigenvalues(x, w, h):
    """Compute the complex eigenvalues

    Args:
        x: initial guess for fsolve
        w: angular fequency
        h: mean water level

    Returns:
        Complex eigenvalues
    """
    eigenvalue = ((w ** 2) * h / 9.8) / x + np.tan(x)
    return eigenvalue


def wave_number(w, h, n, x_0):
    """Dispersion Equation.

    Args:
        w: angular fequency
        h: mean water level
        n: number of complex eigenvalues
        x_0: initial guess for fsolve

    Returns:
        Wave number, evanescent modes
    """
    # initial guess to compute the complex eigenvalues
    j = 1
    x0_complex = np.array([])
    for i in range(n):
        x0_complex = np.append(x0_complex, [j*np.pi / 2 + 0.01])
        j = j + 2

    # compute the complex eigenvalues
    if n != 0:
        x = fsolve(complex_eigenvalues, x0_complex, args=(w, h))
        k_c = abs(x / h)
    else:
        k_c = 0

    # compute the real eigenvalue
    x = fsolve(real_eigenvalue, x_0, args=(w, h))
    k = abs(x / h)
    return k, k_c


# variable initialization
# depth, period, angular frecuency, complex eigenvalues
h = 8
t = 11
w = 2*np.pi / t
n = 5

# initial value for the iteration
x_0 = np.pi

# compute the wave number
k, k_c = wave_number(w, h, n, x_0)
print k, k_c
